@@ -3,7 +3,7 @@ import copy
 import scipy.spatial.transform
 
 
-def ecp(mol, configs, wf, threshold):
+def ecp(mol, configs, wf, threshold, naip=None):
     """
     :returns: ECP value, summed over all the electrons and atoms.
     """
@@ -13,16 +13,18 @@ def ecp(mol, configs, wf, threshold):
         for atom in mol._atom:
             if atom[0] in mol._ecp.keys():
                 for e in range(nelec):
-                    ecp_tot += ecp_ea(mol, configs, wf, e, atom, threshold)["total"]
+                    ecp_tot += ecp_ea(mol, configs, wf, e, atom, threshold, naip)[
+                        "total"
+                    ]
     return ecp_tot
 
 
-def compute_tmoves(mol, configs, wf, e, threshold, tau):
+def compute_tmoves(mol, configs, wf, e, threshold, tau, naip=None):
     """"""
     nconfig = configs.configs.shape[0]
     if mol._ecp != {}:
         data = [
-            ecp_ea(mol, configs, wf, e, atom, threshold)
+            ecp_ea(mol, configs, wf, e, atom, threshold, naip)
             for atom in mol._atom
             if atom[0] in mol._ecp.keys()
         ]
@@ -50,7 +52,7 @@ def compute_tmoves(mol, configs, wf, e, threshold, tau):
     return {"ratio": ratio, "weight": weight, "configs": configs}
 
 
-def ecp_ea(mol, configs, wf, e, atom, threshold):
+def ecp_ea(mol, configs, wf, e, atom, threshold, naip=None):
     """
     :returns: the ECP value between electron e and atom at, local+nonlocal.
     TODO: update documentation
@@ -72,7 +74,7 @@ def ecp_ea(mol, configs, wf, e, atom, threshold):
     # Use masked objects internally
     r_ea = r_ea[mask]
     r_ea_vec = r_ea_vec[mask]
-    P_l, r_ea_i = get_P_l(r_ea, r_ea_vec, l_list)
+    P_l, r_ea_i = get_P_l(r_ea, r_ea_vec, l_list, naip)
 
     # Note: epos_rot is not just apos+r_ea_i because of the boundary;
     # positions of the samples are relative to the electron, not atom.
@@ -82,7 +84,7 @@ def ecp_ea(mol, configs, wf, e, atom, threshold):
     epos_rot[mask] = (configs.configs[mask, e, :] - r_ea_vec)[:, np.newaxis] + r_ea_i
 
     epos = configs.make_irreducible(e, epos_rot, mask)
-    ratio = wf.testvalue(e, epos, mask)
+    ratio, _ = wf.testvalue(e, epos, mask)
 
     # Compute local and non-local parts
     ecp_val[mask] = np.einsum("ij,ik,ijk->i", ratio, masked_v_l, P_l)
@@ -143,7 +145,7 @@ def generate_ecp_functors(coeffs):
 
 
 class rnExp:
-    """
+    r"""
     v_l object.
 
     :math:`cr^{n-2}\cdot\exp(-er^2)`
@@ -188,7 +190,7 @@ def P_l(x, l):
         raise NotImplementedError(f"Legendre functions for l>4 not implemented {l}")
 
 
-def get_P_l(r_ea, r_ea_vec, l_list):
+def get_P_l(r_ea, r_ea_vec, l_list, naip=None):
     r"""The factor :math:`(2l+1)` and the quadrature weights are included.
 
     :parameter r_ea: distances of electron e and atom a
@@ -199,7 +201,8 @@ def get_P_l(r_ea, r_ea_vec, l_list):
     :returns: legendre function P_l values for each :math:`l` channel.
     :rtype: (nconf, naip, nl) array
     """
-    naip = 6 if len(l_list) <= 2 else 12
+    if naip is None:
+        naip = 6 if len(l_list) <= 2 else 12
     nconf = r_ea.shape[0]
     weights, rot_vec = get_rot(nconf, naip)
 
@@ -222,26 +225,76 @@ def get_rot(nconf, naip):
     :rtype:  ((naip,) array, (nconf, naip, 3) array)
     """
 
-    def sphere(t_, p_):
-        s = np.sin(t_)
-        return s * np.cos(p_), s * np.sin(p_), np.cos(t_)
-
     if nconf > 0:  # get around a bug(?) when there are zero configurations.
         rot = scipy.spatial.transform.Rotation.random(nconf).as_matrix()
     else:
         rot = np.zeros((0, 3, 3))
 
-    if naip == 6:
-        d1 = np.array([0.0, 1.0, 0.5, 0.5, 0.5, 0.5]) * np.pi
-        d2 = np.array([0.0, 0.0, 0.0, 0.5, 1.0, 1.5]) * np.pi
-    elif naip == 12:
-        tha = np.arccos(1.0 / np.sqrt(5.0))
-        d1 = np.array([0, np.pi] + [tha, np.pi - tha] * 5)
-        d2 = np.array([0, 0] + list(range(10))) * np.pi / 5
-    else:
+    if naip not in quadrature_grid.keys():
         raise ValueError("Do not support naip!= 6 or 12")
-
-    rot_vec = np.einsum("jil,ik->jkl", rot, sphere(d1, d2))
-    weights = np.ones(naip) / (naip)
-
+    points, weights = quadrature_grid[naip]
+    rot_vec = np.einsum("jkl,ik->jil", rot, points)
     return weights, rot_vec
+
+
+def generate_quadrature_grids():
+    """
+    Generate quadrature grids from Mitas, Shirley, and Ceperley J. Chem. Phys. 95, 3467 (1991)
+        https://doi.org/10.1063/1.460849
+    All the grids in the Mitas paper are hard-coded here.
+    Returns a dictionary whose keys are naip (number of auxiliary points) and whose values are tuples of arrays (points, weights) 
+    """
+    # Generate in Cartesian grids for octahedral symmetry
+    octpts = np.mgrid[-1:2, -1:2, -1:2].reshape(3, -1).T
+    nonzero_count = np.count_nonzero(octpts, axis=1)
+    OA = octpts[nonzero_count == 1]
+    OB = octpts[nonzero_count == 2] / np.sqrt(2)
+    OC = octpts[nonzero_count == 3] / np.sqrt(3)
+    d1 = OC * np.sqrt(3 / 11)
+    d1[:, 2] *= 3
+    OD = np.concatenate([np.roll(d1, i, axis=1) for i in range(3)])
+    OAB = np.concatenate([OA, OB], axis=0)
+    OABC = np.concatenate([OAB, OC], axis=0)
+    OABCD = np.concatenate([OABC, OD], axis=0)
+
+    # Generate in spherical grids for octahedral symmetry
+    def sphere(t_, p_):
+        s = np.sin(t_)
+        return s * np.cos(p_), s * np.sin(p_), np.cos(t_)
+
+    b_1 = np.arctan(2)
+    c_1 = np.arccos((2 + 5 ** 0.5) / (15 + 6 * 5 ** 0.5) ** 0.5)
+    c_2 = np.arccos(1 / (15 + 6 * 5 ** 0.5) ** 0.5)
+    theta, phi = {}, {}
+    theta["A"] = np.array([0, np.pi])
+    phi["A"] = np.zeros(2)
+    k = np.arange(10)
+    theta["B"] = np.tile([b_1, np.pi - b_1], 5)
+    phi["B"] = k * np.pi / 5
+    c_th1 = np.tile([np.pi - c_1, c_1], 5)
+    c_th2 = np.tile([np.pi - c_2, c_2], 5)
+    theta["C"] = np.concatenate([c_th1, c_th2])
+    phi["C"] = np.tile(k * np.pi / 5, 2)
+    I = {g: np.transpose(sphere(theta[g], phi[g])) for g in "ABC"}
+    IAB = np.concatenate([I["A"], I["B"]], axis=0)
+    IABC = np.concatenate([IAB, I["C"]], axis=0)
+
+    lens = {}
+    lens["O"] = [len(x) for x in [OA, OB, OC, OD]]
+    lens["I"] = [len(I[s]) for s in "ABC"]
+
+    def repeat(s, *args):
+        return np.concatenate([np.repeat(w, l) for w, l in zip(args, lens[s])])
+
+    qgrid = {}
+    qgrid[6] = (OA, repeat("O", 1 / 6))
+    qgrid[18] = (OAB, repeat("O", 1 / 30, 1 / 15))
+    qgrid[26] = (OABC, repeat("O", 1 / 21, 4 / 105, 27 / 840))
+    qgrid[50] = (OABCD, repeat("O", 4 / 315, 64 / 2835, 27 / 1280, 14641 / 725760))
+    qgrid[12] = (IAB, repeat("I", 1 / 12, 1 / 12))
+    qgrid[32] = (IABC, repeat("I", 5 / 168, 5 / 168, 27 / 840))
+
+    return qgrid
+
+
+quadrature_grid = generate_quadrature_grids()
