@@ -8,7 +8,7 @@ class JastrowSpin:
 
             The Jastrow form is :math:`e^{U(R)}`, where
 
-        .. math::  U(R) = \sum_{I, i, k} c^{a}_{Ik\sigma(i)} a_{k}(r_{Ii}) + \sum_{i,j,k} c^{b}_{k\sigma(i)\sigma(j)} b^{l}(r_{ij})
+        .. math::  U(R) = \sum_{I, i, k} c^{a}_{Ik\sigma(i)} a_{k}(r_{Ii}) + \sum_{i,j,l} c^{b}_{l\sigma(i)\sigma(j)} b_{l}(r_{ij})
 
 
     """
@@ -55,7 +55,7 @@ class JastrowSpin:
         for e in range(nelec):
             epos = configs.electron(e)
             self._a_partial[e] = self._a_update(e, epos, notmask)
-            self._b_partial[e] = self._b_update(e, epos, notmask)
+            self._b_partial[e], _ = self._b_update(e, epos, notmask)
 
         # electron-electron distances
         nup = self._mol.nelec[0]
@@ -90,7 +90,7 @@ class JastrowSpin:
         u += gpu.cp.einsum("ijkl,jkl->i", self._avalues, self.parameters["acoeff"])
         return (np.ones(len(u)), gpu.asnumpy(u))
 
-    def updateinternals(self, e, epos, wrap=None, mask=None):
+    def updateinternals(self, e, epos, configs, mask=None, saved_values=None):
         r"""Update a and b sums.
         _avalues is the array for current configurations :math:`A_{Iks} = \sum_s a_{k}(r_{Is})` where :math:`s` indexes over :math:`\uparrow` (:math:`\alpha`) and :math:`\downarrow` (:math:`\beta`) sums.
         _bvalues is the array for current configurations :math:`B_{ls} = \sum_s b_{l}(r_{s})` where :math:`s` indexes over :math:`\uparrow\uparrow` (:math:`\alpha_1 < \alpha_2`), :math:`\uparrow\downarrow` (:math:`\alpha, \beta`), and :math:`\downarrow\downarrow` (:math:`\beta_1 < \beta_2`)  sums.
@@ -98,14 +98,17 @@ class JastrowSpin:
         if mask is None:
             mask = [True] * self._configscurrent.configs.shape[0]
         edown = int(e >= self._mol.nelec[0])
-        aupdate = self._a_update(e, epos, mask)
-        bupdate = self._b_update(e, epos, mask)
+        if saved_values is None:
+            aupdate = self._a_update(e, epos, mask)
+            bupdate, bvals = self._b_update(e, epos, mask)
+        else:
+            aupdate, bupdate, bvals = [s[mask] for s in saved_values]
         self._avalues[:, :, :, edown][mask] += aupdate - self._a_partial[e][mask]
         self._bvalues[:, :, edown : edown + 2][mask] += (
             bupdate - self._b_partial[e][mask]
         )
         self._a_partial[e][mask] = aupdate
-        self._update_b_partial(e, epos, mask)
+        self._update_b_partial(e, epos, mask, bvals)
         self._configscurrent.move(e, epos, mask)
 
     def _a_update(self, e, epos, mask):
@@ -148,12 +151,16 @@ class JastrowSpin:
         r = gpu.cp.linalg.norm(d, axis=-1)
         b_partial_e = gpu.cp.zeros((*r.shape[:-1], *self._b_partial.shape[2:]))
 
-        for l, b in enumerate(self.b_basis):
-            bval = b.value(d, r)
-            b_partial_e[..., l, 0] = bval[..., :sep].sum(axis=-1)
-            b_partial_e[..., l, 1] = bval[..., sep:].sum(axis=-1)
+        bvals = gpu.cp.stack([b.value(d, r) for b in self.b_basis], axis=-2)
+        b_partial_e[..., 0] = bvals[..., :sep].sum(axis=-1)
+        b_partial_e[..., 1] = bvals[..., sep:].sum(axis=-1)
 
-        return b_partial_e
+        # for l, b in enumerate(self.b_basis):
+        #    bval = b.value(d, r)
+        #    b_partial_e[..., l, 0] = bval[..., :sep].sum(axis=-1)
+        #    b_partial_e[..., l, 1] = bval[..., sep:].sum(axis=-1)
+
+        return b_partial_e, bvals
 
     def _b_update_many(self, e, epos, mask, spin):
         r"""
@@ -185,7 +192,7 @@ class JastrowSpin:
 
         return b_partial_e
 
-    def _update_b_partial(self, e, epos, mask):
+    def _update_b_partial(self, e, epos, mask, savedbvals):
         r"""
           Calculate b (e-e) partial sum contributions from electron e
         _b_partial_e is the array :math:`B^p_{ils} = \sum_s b_l(r^i_{es}`, with e fixed; :math:`s` indexes over :math:`\uparrow` (:math:`\alpha`) and :math:`\downarrow` (:math:`\beta`) sums, not including electron e.
@@ -200,12 +207,12 @@ class JastrowSpin:
         sep = nup - int(e < nup)
         not_e = np.arange(self._nelec) != e
         edown = int(e >= nup)
-        d = gpu.cp.asarray(
-            epos.dist.dist_i(
-                self._configscurrent.configs[mask][:, not_e], epos.configs[mask]
-            )
-        )
-        r = gpu.cp.linalg.norm(d, axis=-1)
+        # d = gpu.cp.asarray(
+        #    epos.dist.dist_i(
+        #        self._configscurrent.configs[mask][:, not_e], epos.configs[mask]
+        #    )
+        # )
+        # r = gpu.cp.linalg.norm(d, axis=-1)
         dold = gpu.cp.asarray(
             epos.dist.dist_i(
                 self._configscurrent.configs[mask][:, not_e],
@@ -213,14 +220,20 @@ class JastrowSpin:
             )
         )
         rold = gpu.cp.linalg.norm(dold, axis=-1)
-        b_partial_e = gpu.cp.zeros((np.sum(mask), *self._b_partial.shape[2:]))
         eind, mind = np.ix_(not_e, mask)
-        for l, b in enumerate(self.b_basis):
-            bval = b.value(d, r)
-            bdiff = bval - b.value(dold, rold)
-            self._b_partial[eind, mind, l, edown] += bdiff.transpose((1, 0))
-            self._b_partial[e, :, l, 0][mask] = bval[:, :sep].sum(axis=1)
-            self._b_partial[e, :, l, 1][mask] = bval[:, sep:].sum(axis=1)
+
+        oldbvals = gpu.cp.stack([b.value(dold, rold) for b in self.b_basis], axis=-2)
+        bdiff = savedbvals - oldbvals
+        self._b_partial[eind, mind, :, edown] += bdiff.transpose((2, 0, 1))
+        self._b_partial[e, ..., 0][mask] = savedbvals[..., :sep].sum(axis=-1)
+        self._b_partial[e, ..., 1][mask] = savedbvals[..., sep:].sum(axis=-1)
+
+        # for l, b in enumerate(self.b_basis):
+        #    bval = b.value(d, r)
+        #    bdiff = bval - b.value(dold, rold)
+        #    self._b_partial[eind, mind, l, edown] += bdiff.transpose((1, 0))
+        #    self._b_partial[e, :, l, 0][mask] = bval[:, :sep].sum(axis=1)
+        #    self._b_partial[e, :, l, 1][mask] = bval[:, sep:].sum(axis=1)
 
     def value(self):
         """Compute the current log value of the wavefunction"""
@@ -230,7 +243,7 @@ class JastrowSpin:
 
     def gradient(self, e, epos):
         r"""We compute the gradient for electron e as
-        :math:`\nabla_e \ln \Psi_J = \sum_k c_k \left(\sum_{j > e} \nabla_e b_k(r_{ej}) + \sum_{i < e} \nabla_e b_k(r_{ie})\right)`
+        :math:`\nabla_e \ln \Psi_J = \sum_l c_l \left(\sum_{j > e} \nabla_e b_l(r_{ej}) + \sum_{i < e} \nabla_e b_l(r_{ie})\right)`
         So we need to compute the gradient of the b's for these indices.
         Note that we need to compute distances between electron position given and the current electron distances.
         We will need this for laplacian() as well"""
@@ -283,13 +296,16 @@ class JastrowSpin:
         edown = int(e >= nup)
 
         b_partial_e = gpu.cp.zeros((*rnew.shape[:-1], *self._b_partial.shape[2:]))
+        bvals = []
         for l, b in enumerate(self.b_basis):
             c = self.parameters["bcoeff"][l]
             bgrad, bval = b.gradient_value(dnew, rnew)
+            bvals.append(bval)
             grad += c[edown] * gpu.cp.sum(bgrad[:, : nup - eup], axis=1).T
             grad += c[1 + edown] * gpu.cp.sum(bgrad[:, nup - eup :], axis=1).T
             b_partial_e[..., l, 0] = bval[..., : nup - eup].sum(axis=-1)
             b_partial_e[..., l, 1] = bval[..., nup - eup :].sum(axis=-1)
+        bvals = gpu.cp.stack(bvals, axis=-2)
 
         a_partial_e = gpu.cp.zeros((*rinew.shape, self._a_partial.shape[3]))
         for k, a in enumerate(self.a_basis):
@@ -307,7 +323,7 @@ class JastrowSpin:
             "...jk,jk->...", deltab, self.parameters["bcoeff"][:, edown : edown + 2]
         )
         val = gpu.cp.exp(b_val + a_val)
-        return gpu.asnumpy(grad), gpu.asnumpy(val)
+        return gpu.asnumpy(grad), gpu.asnumpy(val), (a_partial_e, b_partial_e, bvals)
 
     def gradient_laplacian(self, e, epos):
         """ """
@@ -359,18 +375,20 @@ class JastrowSpin:
         if mask is None:
             mask = [True] * epos.configs.shape[0]
         edown = int(e >= self._mol.nelec[0])
-        deltaa = self._a_update(e, epos, mask) - self._a_partial[e][mask]
+        aupdate = self._a_update(e, epos, mask)
+        deltaa = aupdate - self._a_partial[e][mask]
         a_val = gpu.cp.einsum(
             "...jk,jk->...", deltaa, self.parameters["acoeff"][..., edown]
         )
-        deltab = self._b_update(e, epos, mask) - self._b_partial[e][mask]
+        bupdate, savedbvals = self._b_update(e, epos, mask)
+        deltab = bupdate - self._b_partial[e][mask]
         b_val = gpu.cp.einsum(
             "...jk,jk->...", deltab, self.parameters["bcoeff"][:, edown : edown + 2]
         )
         val = gpu.cp.exp(b_val + a_val)
         if len(val.shape) == 2:
             val = val.T
-        return gpu.asnumpy(val)
+        return gpu.asnumpy(val), (aupdate, bupdate, savedbvals)
 
     def testvalue_many(self, e, epos, mask=None):
         r"""
