@@ -100,14 +100,14 @@ def vmc_worker(wf, configs, tstep, nsteps, accumulators):
         acc = 0.0
         for e in range(nelec):
             # Propose move
-            g, _ = wf.gradient_value(e, configs.electron(e))
+            g, _, _ = wf.gradient_value(e, configs.electron(e))
             grad = limdrift(np.real(g.T))
             gauss = np.random.normal(scale=np.sqrt(tstep), size=(nconf, 3))
             newcoorde = configs.configs[:, e, :] + gauss + grad * tstep
             newcoorde = configs.make_irreducible(e, newcoorde)
 
             # Compute reverse move
-            g, new_val = wf.gradient_value(e, newcoorde)
+            g, new_val, saved = wf.gradient_value(e, newcoorde)
             new_grad = limdrift(np.real(g.T))
             forward = np.sum(gauss ** 2, axis=1)
             backward = np.sum((gauss + tstep * (grad + new_grad)) ** 2, axis=1)
@@ -119,7 +119,7 @@ def vmc_worker(wf, configs, tstep, nsteps, accumulators):
 
             # Update wave function
             configs.move(e, newcoorde, accept)
-            wf.updateinternals(e, newcoorde, mask=accept)
+            wf.updateinternals(e, newcoorde, configs, mask=accept, saved_values=saved)
             acc += np.mean(accept) / nelec
 
         # Rolling average on step
