@@ -57,7 +57,20 @@ def OPTIMIZE(
     if anchors is None:
         linemin.line_minimization(wf, configs, acc, **linemin_kws)
     else:
-        wfs = [wftools.read_wf(copy.deepcopy(wf), a) for a in anchors]
+        wfs = []
+        for i, a in enumerate(anchors):
+            wfs.append(
+                initialize_qmc_objects(
+                    dft_checkfile,
+                    ci_checkfile=ci_checkfile,
+                    load_parameters=a,
+                    S=S,
+                    jastrow_kws=jastrow_kws,
+                    slater_kws=slater_kws,
+                    target_root=i,
+                )[0]
+            )
+        # wfs = [wftools.read_wf(copy.deepcopy(wf), a) for a in anchors]
         wfs.append(wf)
         optimize_ortho.optimize_orthogonal(wfs, configs, acc, **linemin_kws)
 
@@ -77,12 +90,24 @@ def generate_accumulators(mol, mf, energy=True, rdm1=False, extra_accumulators=N
             raise Exception("Found energy in extra_accumulators and energy is True")
         acc["energy"] = pyqmc.accumulators.EnergyAccumulator(mol)
     if rdm1:
+        if hasattr(mol, "a"):
+            from pyqmc.orbitals import get_k_indices
+
+            kinds = list(set(get_k_indices(mol, mf, supercell.get_supercell_kpts(mol))))
+            kpts = mf.kpts[kinds]
+        else:
+            kpts = None
+
         if "rdm1_up" in acc.keys() or "rdm1_down" in acc.keys():
             raise Exception(
                 "Found rdm1_up or rdm1_down in extra_accumulators and rdm1 is True"
             )
-        acc["rdm1_up"] = obdm.OBDMAccumulator(mol, orb_coeff=mo_coeff[0], spin=0)
-        acc["rdm1_down"] = obdm.OBDMAccumulator(mol, orb_coeff=mo_coeff[1], spin=1)
+        acc["rdm1_up"] = obdm.OBDMAccumulator(
+            mol, orb_coeff=mo_coeff[0], spin=0, kpts=kpts
+        )
+        acc["rdm1_down"] = obdm.OBDMAccumulator(
+            mol, orb_coeff=mo_coeff[1], spin=1, kpts=kpts
+        )
 
     return acc
 
@@ -172,7 +197,9 @@ def initialize_qmc_objects(
 
     configs = pyqmc.mc.initial_guess(mol, nconfig)
     if opt_wf:
-        acc = pyqmc.accumulators.gradient_generator(mol, wf, to_opt, nodal_cutoff=nodal_cutoff)
+        acc = pyqmc.accumulators.gradient_generator(
+            mol, wf, to_opt, nodal_cutoff=nodal_cutoff
+        )
     else:
         if accumulators == None:
             accumulators = {}
@@ -197,7 +224,7 @@ def read_mc_output(
     fname,
     warmup=1,
     reblock=None,
-    exclude_keys=("configs", "weights", "block", "nconfig"),
+    exclude_keys=("configs", "weights", "block", "nconfig", "wrap"),
 ):
     ret = {"fname": fname, "warmup": warmup, "reblock": reblock}
     with h5py.File(fname, "r") as f:
