@@ -2,6 +2,7 @@ import numpy as np
 import pyqmc.gpu as gpu
 import pyqmc.determinant_tools as determinant_tools
 import pyqmc.orbitals
+import warnings
 
 
 def sherman_morrison_row(e, inv, vec):
@@ -124,7 +125,7 @@ class Slater:
             self._det_map,
             self.orbitals,
         ) = pyqmc.orbitals.choose_evaluator_from_pyscf(
-            mol, mf, mc, twist=twist, determinants=determinants
+            mol, mf, mc, twist=twist, determinants=determinants, tol=self.tol
         )
 
         self.parameters = JoinParameters([self.myparameters, self.orbitals.parameters])
@@ -152,29 +153,53 @@ class Slater:
             self._dets.append(
                 gpu.cp.asarray(np.linalg.slogdet(mo_vals))
             )  # Spin, (sign, val), nconf, [ndet_up, ndet_dn]
-            self._inverse.append(
-                gpu.cp.linalg.inv(mo_vals)
-            )  # spin, Nconf, [ndet_up, ndet_dn], nelec, nelec
+
+            is_zero = np.sum(np.abs(self._dets[s][0]) < 1e-16)
+            compute = np.isfinite(self._dets[s][1])
+            if is_zero > 0:
+                warnings.warn(
+                    f"A wave function is zero. Found this proportion: {is_zero/nconf}"
+                )
+                # print(configs.configs[])
+                print(f"zero {is_zero/np.prod(compute.shape)}")
+            self._inverse.append(np.zeros(mo_vals.shape, dtype=mo_vals.dtype))
+            for d in range(compute.shape[1]):
+                self._inverse[s][compute[:, d], d, :, :] = gpu.cp.linalg.inv(
+                    mo_vals[compute[:, d], d, :, :]
+                )
+            # spin, Nconf, [ndet_up, ndet_dn], nelec, nelec
         return self.value()
 
-    def updateinternals(self, e, epos, mask=None):
+    def updateinternals(self, e, epos, configs, mask=None, saved_values=None):
         """Update any internals given that electron e moved to epos. mask is a Boolean array
         which allows us to update only certain walkers"""
 
         s = int(e >= self._nelec[0])
         if mask is None:
-            mask = [True] * epos.configs.shape[0]
-        eeff = e - s * self._nelec[0]
-        ao = self.orbitals.aos("GTOval_sph", epos, mask)
-        self._aovals[:, mask, e, :] = ao
-        mo = self.orbitals.mos(ao, s)
+            mask = np.ones(epos.configs.shape[0], dtype=bool)
+        is_zero = np.sum(np.isinf(self._dets[s][1]))
+        if is_zero:
+            warnings.warn(
+                "Found a zero in the wave function. Recomputing everything. This should not happen often."
+            )
+            self.recompute(configs)
+            return
 
+        eeff = e - s * self._nelec[0]
+        if saved_values is None:
+            ao = self.orbitals.aos("GTOval_sph", epos, mask)
+            self._aovals[:, mask, e, :] = ao
+            mo = self.orbitals.mos(ao, s)
+        else:
+            ao, mo = saved_values
+            self._aovals[:, mask, e, :] = ao[:, mask]
+            mo = mo[mask]
         mo_vals = mo[:, self._det_occup[s]]
         det_ratio, self._inverse[s][mask, :, :, :] = sherman_morrison_ms(
             eeff, self._inverse[s][mask, :, :, :], mo_vals
         )
-
-        self._updateval(det_ratio, s, mask)
+        self._dets[s][0, mask, :] *= self.get_phase(det_ratio)
+        self._dets[s][1, mask, :] += gpu.cp.log(gpu.cp.abs(det_ratio))
 
     def value(self):
         """Return logarithm of the wave function as noted in recompute()"""
@@ -184,10 +209,6 @@ class Slater:
             updets, dndets, self.parameters["det_coeff"]
         )
 
-    def _updateval(self, ratio, s, mask):
-        self._dets[s][0, mask, :] *= self.get_phase(ratio)
-        self._dets[s][1, mask, :] += gpu.cp.log(gpu.cp.abs(ratio))
-
     def _testrow(self, e, vec, mask=None, spin=None):
         """vec is a nconfig,nmo vector which replaces row e"""
         s = int(e >= self._nelec[0]) if spin is None else spin
@@ -219,11 +240,7 @@ class Slater:
             self.parameters["det_coeff"],
             det_array,
         )
-        denom = gpu.cp.einsum(
-            "d,di->i...",
-            self.parameters["det_coeff"],
-            det_array,
-        )
+        denom = gpu.cp.einsum("d,di->i...", self.parameters["det_coeff"], det_array,)
 
         if len(numer.shape) == 2:
             denom = denom[:, gpu.cp.newaxis]
@@ -258,11 +275,7 @@ class Slater:
             self.parameters["det_coeff"],
             det_array,
         )
-        denom = gpu.cp.einsum(
-            "d,di->i...",
-            self.parameters["det_coeff"],
-            det_array,
-        )
+        denom = gpu.cp.einsum("d,di->i...", self.parameters["det_coeff"], det_array,)
         # curr_val = self.value()
 
         if len(numer.shape) == 3:
@@ -301,10 +314,14 @@ class Slater:
         mograd_vals = mograd[:, :, self._det_occup[s]]
 
         ratios = gpu.asnumpy(self._testrowderiv(e, mograd_vals))
-        return ratios[1:] / ratios[0], ratios[0]
+        derivatives = ratios[1:] / ratios[0]
+        derivatives[~np.isfinite(derivatives)] = 0.0
+        values = ratios[0]
+        values[~np.isfinite(values)] = 1.0
+        return derivatives, values, (aograd[:, 0], mograd[0])
 
     def laplacian(self, e, epos):
-        """ Compute the laplacian Psi/ Psi. """
+        """Compute the laplacian Psi/ Psi."""
         s = int(e >= self._nelec[0])
         ao = self.orbitals.aos("GTOval_sph_deriv2", epos)
         ao_val = ao[:, 0, :, :]
@@ -338,7 +355,7 @@ class Slater:
             mo_vals = mo_vals.reshape(
                 -1, epos.configs.shape[1], mo_vals.shape[1], mo_vals.shape[2]
             )
-        return gpu.asnumpy(self._testrow(e, mo_vals, mask))
+        return gpu.asnumpy(self._testrow(e, mo_vals, mask)), (ao, mo)
 
     def testvalue_many(self, e, epos, mask=None):
         """return the ratio between the current wave function and the wave function if
@@ -377,16 +394,25 @@ class Slater:
 
         # Det coeff
         curr_val = self.value()
-        d["det_coeff"] = (
-            self._dets[0][0, :, self._det_map[0]]
-            * self._dets[1][0, :, self._det_map[1]]
+        nonzero = curr_val[0] != 0.0
+
+        # dets[spin][ (phase,log), configuration, determinant]
+        dets = (
+            self._dets[0][:, :, self._det_map[0]],
+            self._dets[1][:, :, self._det_map[1]],
+        )
+
+        d["det_coeff"] = np.zeros(dets[0].shape[1:], dtype=dets[0].dtype)
+        d["det_coeff"][nonzero, :] = (
+            dets[0][0, nonzero, :]
+            * dets[1][0, nonzero, :]
             * gpu.cp.exp(
-                self._dets[0][1, :, self._det_map[0]]
-                + self._dets[1][1, :, self._det_map[1]]
-                - gpu.cp.array(curr_val[1])
+                dets[0][1, nonzero, :]
+                + dets[1][1, nonzero, :]
+                - gpu.cp.array(curr_val[1][nonzero, np.newaxis])
             )
-            / gpu.cp.array(curr_val[0])
-        ).T
+            / gpu.cp.array(curr_val[0][nonzero, np.newaxis])
+        )
 
         for s, parm in zip([0, 1], ["mo_coeff_alpha", "mo_coeff_beta"]):
             ao = self._aovals[
